@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -62,8 +62,7 @@ function was aborted or not-a-number error if an invalid number was read. */
 
 
 
-/* This is the dispatcher of all actions that have some effect on the text are
-	dispatched. 
+/* This is the dispatcher of all actions that have some effect on the text.
 
 	The arguments are an action to be executed, a possible integer parameter and
 	a possible string parameter. -1 and NULL are, respectively, reserved values
@@ -402,6 +401,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		SET_USER_FLAG(b, c, opt.tabs);
 		return OK;
 
+	case SHIFTTABS_A:
+		SET_USER_FLAG(b, c, opt.shift_tabs);
+		return OK;
+
 	case AUTOMATCHBRACKET_A:
 		if (c < 0 && (c = request_number("Match mode (sum of 0:none, 1:brightness, 2:inverse, 4:bold, 8:underline)", b->opt.automatch))<0||c>15) return ((c) == ABORT ? OK : INVALID_MATCH_MODE);
 		b->opt.automatch = c;
@@ -1043,8 +1046,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 	case TABSIZE_A:
 		if (c < 0 && (c = request_number("TAB Size", b->opt.tab_size))<=0) return NUMERIC_ERROR(c);
 		if (c < ne_columns / 2) {
+			i = b->cur_pos;
 			move_to_sol(b);
 			b->opt.tab_size = c;
+			goto_pos(b,i);
 			reset_window();
 			return OK;
 		}
@@ -1163,7 +1168,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	case DOUNDO_A:
 		SET_USER_FLAG(b, c, opt.do_undo);
-		if (!(b->opt.do_undo)) reset_undo_buffer(&b->undo);
+		if (!(b->opt.do_undo)) {
+			reset_undo_buffer(&b->undo);
+			b->atomic_undo = 0;
+		}
 		return OK;
 
 	case READONLY_A:
@@ -1180,6 +1188,26 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		b->find_string_changed = 1;
 		return OK;
 
+	case ATOMICUNDO_A:
+		if (b->opt.do_undo) {
+			/* set c to the desired b->link_undos */
+			if (!p) {
+				c = b->link_undos ? b->link_undos - 1 : 1;
+			} else if (p[0]=='0') {
+				c = 0;
+			} else if (p[0]=='-') {
+				c = b->link_undos ? b->link_undos - 1 : 0;
+			} else if (p[0]=='+' || p[0]=='1') {  /* Kindly allow undocumented "AtomicUndo 1" also. */
+				c = b->link_undos + 1;
+			} else return INVALID_LEVEL;
+			while(c > b->link_undos) start_undo_chain(b);
+			while(c < b->link_undos) end_undo_chain(b);
+			b->atomic_undo = (c > 0) ? 1 : 0;
+			snprintf(msg, MAX_MESSAGE_SIZE, "AtomicUndo level: %d", c);
+			print_message(msg);
+			return OK;
+		} else return UNDO_NOT_ENABLED;
+
 	case RECORD_A:
 		recording = b->recording;
 		SET_USER_FLAG(b, c, recording);
@@ -1281,12 +1309,13 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	case MARK_A:
 	case MARKVERT_A:
+		if (c < 0) c = 1;
 		SET_USER_FLAG(b, c, marking);
 		if (!b->marking) return(OK);
 		print_message(info_msg[a==MARK_A ? BLOCK_START_MARKED : VERTICAL_BLOCK_START_MARKED]);
 		b->mark_is_vertical = (a == MARKVERT_A);
 		b->block_start_line = b->cur_line;
-		b->block_start_col = b->win_x + b->cur_x;
+		b->block_start_pos = b->cur_pos;
 		return OK;
 
 	case CUT_A:
@@ -1318,7 +1347,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (b->marking) {
 			delay_update();
 			goto_line(b, b->block_start_line);
-			goto_column(b, b->block_start_col);
+			goto_column(b, calc_width(b->cur_line_desc, b->block_start_pos, b->opt.tab_size, b->encoding));
 			return OK;
 		}
 		print_error(MARK_BLOCK_FIRST);
@@ -1494,6 +1523,14 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
 
+	case SHIFT_A:
+		if (b->opt.read_only) return FILE_IS_READ_ONLY;
+
+		error = shift(b, p, &msg[0], MAX_MESSAGE_SIZE);
+		if (stop) error = STOPPED;
+		if (p) free(p);
+		return print_error(error) ? ERROR : 0;
+
 	case LOADPREFS_A:
 		if (p || (p = request_file(b, "Prefs Name", NULL))) {
 			error = print_error(load_prefs(b, p));
@@ -1541,6 +1578,12 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		NORMALIZE(c);
 		delay_update();
 
+		if (b->atomic_undo) {
+			b->atomic_undo = 0;
+			while (b->link_undos) end_undo_chain(b);
+			print_message("AtomicUndo level: 0");
+		}
+
 		for(i = 0; i < c && !(error = undo(b)) && !stop; i++);
 		if (stop) error = STOPPED;
 		b->is_modified = b->undo.cur_step != b->undo.last_save_step;
@@ -1581,24 +1624,13 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			we do not want to record this insertion twice. Also, we are counting on 
 			INSERTSTRING_A to handle character encoding issues. */
 		recording = b->recording;
+
 		i = b->cur_pos;
 		
 		if ( !p ) { /* no prefix given; find one left of the cursor. */
-			if (i && i <= b->cur_line_desc->line_len) {
-				i = prev_pos(b->cur_line_desc->line, i, b->encoding);
-				while (i && ne_isword(c = get_char(&b->cur_line_desc->line[i], b->encoding), b->encoding))
-					i = prev_pos(b->cur_line_desc->line, i, b->encoding);
-				if (! ne_isword(c = get_char(&b->cur_line_desc->line[i], b->encoding), b->encoding))
-					i = next_pos(b->cur_line_desc->line, i, b->encoding);
-				p = malloc(b->cur_pos - i + 1);
-				if (!p) return OUT_OF_MEMORY;
-				strncpy(p, &b->cur_line_desc->line[i], b->cur_pos - i);
-			} 
-			else p = malloc(1); /* no prefix left of the cursor; we'll find _all_ word strings! */
-			
-			p[b->cur_pos - i] = 0;
+			if ( context_prefix(b, &p, &i, b->encoding) ) return OUT_OF_MEMORY;
 		}	
-
+      
 		snprintf(msg, MAX_MESSAGE_SIZE, "AutoComplete: prefix \"%s\"", p);
 			
 		if (p = autocomplete(p, msg, TRUE, &col)) {
