@@ -1,7 +1,7 @@
 /* Buffer handling functions, including allocation, deallocation, and I/O.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -185,6 +185,7 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 
 		b->opt.insert         =
 		b->opt.tabs           =
+		b->opt.shift_tabs     =
 		b->opt.automatch      =
 		b->opt.do_undo        =
 		b->opt.auto_prefs     = 1;
@@ -198,6 +199,7 @@ buffer *alloc_buffer(const buffer * const cur_b) {
 			b->opt.cur_clip       = cur_b->opt.cur_clip;
 			b->opt.tab_size       = cur_b->opt.tab_size;
 			b->opt.tabs           = cur_b->opt.tabs;
+			b->opt.shift_tabs     = cur_b->opt.shift_tabs;
 			b->opt.automatch      = cur_b->opt.automatch;
 			b->opt.right_margin   = cur_b->opt.right_margin;
 
@@ -829,7 +831,7 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 			b->is_modified = 1;
 			
 			/* We just inserted len chars at (line,pos); adjust bookmarks and mark accordingly. */
-			if (b->marking && b->block_start_line == line && b->block_start_col > pos) b->block_start_col += len;
+			if (b->marking && b->block_start_line == line && b->block_start_pos > pos) b->block_start_pos += len;
 
 			for (i=0, mask=b->bookmark_mask;  mask;  i++, mask >>= 1) 
 				if ((mask & 1) && b->bookmark[i].line == line && b->bookmark[i].pos > pos) b->bookmark[i].pos += len;
@@ -859,8 +861,8 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 				/* We just inserted a line break at (line,pos);
 				   adjust the buffer bookmarks and mark accordingly. */
 				if (b->marking) {
-					if (b->block_start_line == line && b->block_start_col > pos) {
-						b->block_start_col -= pos;
+					if (b->block_start_line == line && b->block_start_pos > pos) {
+						b->block_start_pos -= pos;
 						b->block_start_line++;
 					}
 					else if (b->block_start_line > line) b->block_start_line++;
@@ -883,9 +885,6 @@ int insert_stream(buffer * const b, line_desc * ld, int line, int pos, const uns
 			}
 		}
 		
-		
-		
-
 		s += len + 1;
 	}
 
@@ -946,7 +945,6 @@ int insert_spaces(buffer * const b, line_desc * const ld, const int line, const
 	undo buffer. */
 
 int delete_stream(buffer * const b, line_desc * const ld, const int line, const int pos, int len) {
-
 	int n, m, mask;
 
 	assert_buffer(b);
@@ -959,7 +957,7 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 	block_signals();
 
 	if (b->opt.do_undo && !(b->undoing || b->redoing)) {
-		int error = add_undo_step(b, line, pos, len);
+		const int error = add_undo_step(b, line, pos, len);
 		if (error) {
 			release_signals();
 			return error;
@@ -967,7 +965,6 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 	}
 
 	while(len) {
-
 		/* First case: we are just on the end of a line. We join the current
 		line with the following one (if it's there of course). If, however,
 		the current line is empty, we rather remove it. The only difference
@@ -976,15 +973,14 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 		if (pos == ld->line_len) {
 			unsigned char *p;
 			line_desc *next_ld = (line_desc *)ld->ld_node.next;
-
-			assert(next_ld != NULL);
-			assert_line_desc(next_ld, b->encoding);
+			/* There's nothing more to do--we are at the end of the file. */
+			if (next_ld->ld_node.next == NULL) break;
 
 			/* We're about to join line+1 to line; adjust mark and bookmarks accordingly. */
 			if (b->marking) {
 				if (b->block_start_line == line+1) {
 					b->block_start_line--;
-					b->block_start_col += ld->line_len;
+					b->block_start_pos += ld->line_len;
 				}
 				else if (b->block_start_line > line) b->block_start_line--;
 			}
@@ -1020,6 +1016,7 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 				}
 				else {
 					release_signals();
+					if (b->opt.do_undo && !(b->undoing || b->redoing)) fix_last_undo_step(b, -len);
 					return OUT_OF_MEMORY;
 				}
 			}
@@ -1057,10 +1054,10 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 			}
 		}
 
-		/* Second case: we are inside a line. We delete len characters or, if
-			there are less then len characters to delete, we delete up to the end
+		/* Second case: we are inside a line. We delete len bytes or, if
+			there are less then len bytes to delete, we delete up to the end
 			of the line. In the latter case, we simply set the line length and
-			free the corresponding characters.  Otherwise, the number of characters to
+			free the corresponding bytes.  Otherwise, the number of bytes to
 			move is minimized. */
 
 		else {
@@ -1069,11 +1066,11 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 			/* We're about to erase n chars at (line,pos); adjust mark and bookmarks accordingly. */
 			if (b->marking)
 				if (b->block_start_line == line)
-					if (b->block_start_col >= pos)
-						if (b->block_start_col < pos + n)
-							b->block_start_col = pos;
+					if (b->block_start_pos >= pos)
+						if (b->block_start_pos < pos + n)
+							b->block_start_pos = pos;
 						else
-							b->block_start_col -= n;
+							b->block_start_pos -= n;
 			for (m=0, mask=b->bookmark_mask;  mask;  m++, mask>>=1) {
 				if (mask & 1) {
 					if (b->bookmark[m].line == line)
@@ -1109,6 +1106,8 @@ int delete_stream(buffer * const b, line_desc * const ld, const int line, const
 		b->is_modified = 1;
 	}
 
+	if (b->opt.do_undo && !(b->undoing || b->redoing)) fix_last_undo_step(b, -len);
+
 	release_signals();
 	return OK;
 }
