@@ -1,26 +1,25 @@
 /* Optimal cursor motion definitions.
+	Based primarily on public domain code written by Chris Torek.
+	Originally part of GNU Emacs.	Vastly edited and modified for use within ne.
 
 	Copyright (C) 1985, 1989 Free Software Foundation, Inc.
-	Based primarily on public domain code written by Chris Torek
-	Originally part of GNU Emacs.
-	Vastly edited and modified for use within ne.
+	Copyright (C) 1993-1998 Sebastiano Vigna 
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the Free
-	Software Foundation; either version 2, or (at your option) any later
-	version.
-	
-	This program is distributed in the hope that it will be useful, but WITHOUT
-	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-	more details.
-	
-	You should have received a copy of the GNU Library General Public License
-	along with the GNU C Library; see the file COPYING.LIB.  If not, write to
-	the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+	This library is free software; you can redistribute it and/or modify it
+	under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 3 of the License, or (at your
+	option) any later version.
+
+	This library is distributed in the hope that it will be useful, but
+	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 /* This structure holds everything needed to do cursor motion. */
