@@ -1,7 +1,7 @@
 /* Command table manipulation functions and vectors.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -62,6 +62,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(ABOUT         ), NO_ARGS                                                               },
 	{ NAHL(ADJUSTVIEW    ),           ARG_IS_STRING                                               },
 	{ NAHL(ALERT         ), NO_ARGS                                                               },
+	{ NAHL(ATOMICUNDO    ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(AUTOCOMPLETE  ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(AUTOINDENT    ),                           IS_OPTION                                   },
 	{ NAHL(AUTOMATCHBRACKET),                         IS_OPTION                                   },
@@ -170,6 +171,8 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(SEARCHBACK    ),                           IS_OPTION                                   },
 	{ NAHL(SELECTDOC     ),0                                                                      },
 	{ NAHL(SETBOOKMARK   ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
+	{ NAHL(SHIFT         ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
+	{ NAHL(SHIFTTABS     ),                           IS_OPTION                                   },
 	{ NAHL(STATUSBAR     ),                           IS_OPTION                                   },
 	{ NAHL(SUSPEND       ), NO_ARGS                                                               },
 	{ NAHL(SYNTAX        ),           ARG_IS_STRING | IS_OPTION                                   },
@@ -277,7 +280,13 @@ action parse_command_line(const unsigned char * command_line, int * const num_ar
 
 	command_line = p;
 
-	if (!isalpha(*p)) return NOP_A;
+	if (!isalpha(*p)) { /* Comment, treated as NOP. */
+		int len = strlen(p);
+		if (!(*string_arg = malloc(len + 1))) return -OUT_OF_MEMORY;
+		memcpy(*string_arg, p, len);
+		(*string_arg)[len] = 0;
+		return NOP_A;
+	}
 
 	while(*p && !isasciispace(*p)) p++;
 
@@ -385,6 +394,14 @@ void record_action(char_stream *cs, action a, int c, unsigned char *p, int verbo
 	char t[MAX_INT_LEN + 2];
 
 	if (commands[a].flags & DO_NOT_RECORD) return;
+	
+	/* NOP_A is special; it may actually be a comment.
+	   Blank lines and real NOPs are recorded as blank lines. */
+	if (a == NOP_A) {
+		if (p && *p) add_to_stream(cs, p, strlen(p) + 1);
+		else add_to_stream(cs, "", 1);
+		return;
+	}
 
 	if (verbose) add_to_stream(cs, commands[a].name, strlen(commands[a].name));
 	else add_to_stream(cs, commands[a].short_name, strlen(commands[a].short_name));
@@ -445,9 +462,9 @@ static int insertchar_val(const unsigned char *p) {
 
 void optimize_macro(char_stream *cs, int verbose) {
 	char *cmd;
-	int	pos;
-	int	chr;
-	int	building = 0;
+	int pos;
+	int chr;
+	int building = 0;
 	 
 	if (!cs || !cs->len) return;
 
@@ -464,14 +481,14 @@ void optimize_macro(char_stream *cs, int verbose) {
 			}
 			else {
 				const char *insert;
-				int	len;
+				int len;
 				insert = verbose ? INSERTSTRING_NAME : INSERTSTRING_ABBREV;
-				len	 = strlen(insert);
-				insert_in_stream(cs, "\"",	pos, 2);	/* Closing quote */
-				insert_in_stream(cs, two,	 pos, 1);		/* The character itself */
-				insert_in_stream(cs, " \"",  pos, 2);	/* space and openning quote */
-				insert_in_stream(cs, insert, pos, len);	/* The command itself */
-				building = pos + len + 2;						/* This is where the char is now */
+				len  = strlen(insert);
+				insert_in_stream(cs, "\"",   pos, 2);   /* Closing quote */
+				insert_in_stream(cs, two,    pos, 1);   /* The character itself */
+				insert_in_stream(cs, " \"",  pos, 2);   /* space and openning quote */
+				insert_in_stream(cs, insert, pos, len); /* The command itself */
+				building = pos + len + 2;               /* This is where the char is now */
 			}
 		}
 		else {
@@ -505,6 +522,7 @@ int play_macro(buffer *b, char_stream *cs) {
 
 	stop = FALSE;
 
+	b->executing_macro = 1;
 	while(!stop && p - stream < len) {	
 #ifdef NE_TEST
 		fprintf(stderr, "%s\n", p); /* During tests, we output to stderr the current command. */
@@ -564,6 +582,9 @@ macro_desc *load_macro(const char *name) {
 	}
 
 	if (cs) {
+		/* the last line may not be null-terminated, so... */
+		add_to_stream(cs, "", 1);
+
 		md->cs = cs;
 		md->name = str_dup(file_part(name));
 
@@ -589,10 +610,16 @@ macro list, it is loaded. A standard error code is returned. */
 
 int execute_macro(buffer *b, const char *name) {
 
+	static call_depth = 0;
 	const char *p;
 	macro_desc *md;
 	int h;
 
+	if (++call_depth > 32) {
+		--call_depth;
+		return MAX_MACRO_DEPTH_EXCEEDED;
+	}
+
 	p = file_part(name);
 
 	h = hash_macro(p, strlen(p));
@@ -605,8 +632,21 @@ int execute_macro(buffer *b, const char *name) {
 
 	assert_macro_desc(md);
 
-	if (md) return play_macro(b, md->cs);
+	if (md) {
+		if (b->recording) {
+			add_to_stream(b->cur_macro, "# include macro ", 16);
+			add_to_stream(b->cur_macro, md->name, strlen(md->name)+1);
+		}
+		h = play_macro(b, md->cs);
+		if (b->recording) {
+			add_to_stream(b->cur_macro, "# conclude macro ", 17);
+			add_to_stream(b->cur_macro, md->name, strlen(md->name)+1);
+		}
+		--call_depth;
+		return h;
+	}
 
+	--call_depth;
 	return CANT_OPEN_MACRO;
 }
 
@@ -631,7 +671,40 @@ void unload_macros(void) {
 	}
 }
 
-
+/* Find any key strokes that currently map to commands[i].name or commands[i].short_name.
+   Returns either NULL or a char string that must be freed by the caller. */
+char *find_key_strokes(int c) {
+	int i;
+	char *str=NULL, *p;
+	
+	for (i=0; i<NUM_KEYS; i++) {
+		if (key_binding[i]) {
+			if (((!strncasecmp(commands[c].short_name,key_binding[i],strlen(commands[c].short_name))) &&
+			     ((!key_binding[i][strlen(commands[c].short_name)]      ) || 
+			      (key_binding[i][strlen(commands[c].short_name)] == ' ')
+			     )
+			    ) || 
+			    ((!strncasecmp(commands[c].name,key_binding[i],strlen(commands[c].name))) &&
+			     ((!key_binding[i][strlen(commands[c].name)]      ) ||
+			      (key_binding[i][strlen(commands[c].name)] == ' ')
+			     )
+			    )
+			   ) {
+				if (!str) {
+					if (!(str = malloc(14))) return NULL;
+					strcpy(str,"Bound key(s):");
+				}
+				if (p = realloc(str,strlen(str) + strlen(key_stroke[i]) + 2)) {
+					str = strcat(strcat(p," "),key_stroke[i]);
+				} else {
+					free(str);
+					return NULL;
+				}
+			}
+		}
+	}
+	return str;
+}
 
 /* This function helps. The help text relative to the command name pointed to
 	by p is displayed (p can also contain arguments). If p is NULL, the
@@ -641,6 +714,8 @@ void unload_macros(void) {
 void help(char *p) {
 
 	action a;
+	char *key_strokes;
+	char **tmphelp;
 	int j, i = 0;
 	D(fprintf(stderr,"Help Called with parm %p.\n",p);)
 	do {
@@ -672,7 +747,18 @@ void help(char *p) {
 			assert(i >= 0 && i < ACTION_COUNT);
 
 			print_message("Help: press Enter, or F1 or Escape or Escape-Escape");
-			if ((j = request_strings(commands[i].help, commands[i].help_len, 0, ne_columns, FALSE)) < 0 ) i = j;
+			if ((key_strokes = find_key_strokes(i)) && (tmphelp = calloc(commands[i].help_len+1, sizeof(char *)))) {
+				tmphelp[0] = (char *)commands[i].help[0];
+				tmphelp[1] = (char *)commands[i].help[1];
+				tmphelp[2] = key_strokes;
+				memcpy(&tmphelp[3], &commands[i].help[2], sizeof(char *) * (commands[i].help_len-2)); 
+				if ((j = request_strings((const char * const* const)tmphelp, commands[i].help_len+1, 0, ne_columns, FALSE)) < 0 ) i = j;
+				free(tmphelp);
+			} else {
+				if ((j = request_strings(commands[i].help, commands[i].help_len, 0, ne_columns, FALSE)) < 0 ) i = j;
+			}
+			if (key_strokes)
+				free(key_strokes);
 		}
 	} while(i >= 0);
 	draw_status_bar();
