@@ -77,6 +77,8 @@ static struct e2s const e2s[] = {
 	{ "sage", "python" },
 	{ "sty", "tex" },
 	{ "tcsh", "csh" },
+	{ "texi", "texinfo" },
+	{ "txi", "texinfo" },
 	{ "v", "verilog" },
 	{ "vh", "verilog" },
 	{ "vhd", "verilog" },
