@@ -1,30 +1,30 @@
 /* Help strings.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+	This library is free software; you can redistribute it and/or modify it
+	under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 3 of the License, or (at your
+	option) any later version.
 
-const char * const ABOUT_HELP[5] = {
+	This library is distributed in the hope that it will be useful, but
+	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+
+const char * const ABOUT_HELP[7] = {
   "Syntax: About",
   "Abbreviation: About",
   "",
-  "displays a simple information line about `ne' on the status bar.",
+  "displays the copyright splash screen and places a simple information",
+  "line containing the version and build date of `ne' on the status bar.",
+  "Press any key to dismiss this screen.",
 };
 
 const char * const ADJUSTVIEW_HELP[26] = {
@@ -62,6 +62,30 @@ const char * const ALERT_HELP[5] = {
   "beeps or flashes, depending on the value of the visual bell flag.",
 };
 
+const char * const ATOMICUNDO_HELP[22] = {
+  "Syntax: AtomicUndo [0|+|-]",
+  "Abbreviation: AU",
+  "",
+  "increases, decreases, sets or clears the `AtomicUndo' level.  The",
+  "normal level is zero. All current document changes made while the",
+  "`AtomicUndo' level is above zero are treated as a single change by the",
+  "`Undo' and `Redo' commands. If no parameter is given, a level of 0 is",
+  "set to 1; otherwise the current non-zero level is decremented. If 0 is",
+  "given, the level is reset to zero. Parameters of `+' and `-'",
+  "respectively increment and decrement the level, which in no case can be",
+  "negative. If the level is above zero, the `DoUndo' flag in the status",
+  "bar, which is normally a lower-case `u', becomes upper case `U'.",
+  "",
+  "   Two other actions will reset the `AtomicUndo' level to zero:",
+  "invoking the `Undo' command, and disabling the undo system with the",
+  "`DoUndo' command. You cannot set a non-zero `AtomicUndo' level unless",
+  "the undo system is enabled.",
+  "",
+  "   Note: macros that you wish to undo and redo atomically--i.e., as if",
+  "they were single commands--should begin with `AtomicUndo +' and end with",
+  "`AtomicUndo -' so that they can call and/or be called by other macros.",
+};
+
 const char * const AUTOCOMPLETE_HELP[20] = {
   "Syntax: AutoComplete [PREFIX]",
   "Abbreviation: AC",
@@ -108,6 +132,19 @@ const char * const AUTOINDENT_HELP[22] = {
   "`Undo'.",
 };
 
+const char * const AUTOMATCHBRACKET_HELP[11] = {
+  "Syntax: AutoMatchBracket [0..15]",
+  "Abbreviation: AMB",
+  "",
+  "sets the auto match bracket mode. When the cursor is on a recognized",
+  "bracket (`{}', `()', `[]' or `<>') and the associated matching bracket",
+  "is on the screen, that matching bracket will be indicated according to",
+  "the mode. The mode is either zero for no bracket matching, or the sum",
+  "of 1 (altered foreground and background brightness), 2 (inverse), 4",
+  "(bold), and 8 (underline). If no mode is specified, `ne' prompts your",
+  "for one. The default mode is 1. See `MatchBracket'.",
+};
+
 const char * const AUTOPREFS_HELP[14] = {
   "Syntax: AutoPrefs [0|1]",
   "Abbreviation: AP",
@@ -317,7 +354,7 @@ const char * const DELETEPREVWORD_HELP[8] = {
   "word the effect is just to delete to the start of that word).",
 };
 
-const char * const DOUNDO_HELP[20] = {
+const char * const DOUNDO_HELP[21] = {
   "Syntax: DoUndo [0|1]",
   "Abbreviation: DU",
   "",
@@ -328,7 +365,8 @@ const char * const DOUNDO_HELP[20] = {
   "   If you invoke `DoUndo' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
   "respectively. A lower case `u' will appear on the status bar if the",
-  "flag is true.",
+  "flag is true. (The `U' will be upper case if the flag is true and the",
+  "`AtomicUndo' level is non-zero.)",
   "",
   "   The usefulness of this option relies in the fact that the undo",
   "system is a major memory eater. If you plan to do massive editing (say,",
@@ -446,7 +484,7 @@ const char * const FINDREGEXP_HELP[12] = {
   "on the input line, the default being the last pattern used.",
 };
 
-const char * const FLAGS_HELP[30] = {
+const char * const FLAGS_HELP[33] = {
   "Syntax: Flags",
   "Abbreviation: FLAG",
   "",
@@ -465,6 +503,7 @@ const char * const FLAGS_HELP[30] = {
   "       u   DoUndo        DU   record edits for later undoing",
   "       r   ReadOnly      RO   changes are not allowed",
   "       t   Tabs          TAB  TAB key inserts TABs instead of spaces",
+  "       T   ShiftTabs     ST   Shift may insert TABs",
   "       B   Binary        B    affects file loading/saving",
   "       M   Mark          M    mark set for line-oriented block operations",
   "       V   MarkVert      MV   like mark, but block is rectangle",
@@ -474,8 +513,10 @@ const char * const FLAGS_HELP[30] = {
   "       *   Modified      MOD  document has been modified since last saved",
   "       @   UTF8IO        U8IO I/O (keyboard and terminal) are UTF-8 encoded",
   "     A/8/U UTF8          U8   the document encoding (ASCII, 8-bit or UTF-8)",
-  "   The `RequestOrder' flag's state is not indicated on the status bar.",
-  "See `RequestOrder'.",
+  "",
+  "   The `RequestOrder' and `AutoMatchBracket' flags' states are not",
+  "indicated on the status bar.  See `RequestOrder' and ",
+  "`AutoMatchBracket' respectively.",
 };
 
 const char * const FLASH_HELP[6] = {
@@ -506,19 +547,21 @@ const char * const FREEFORM_HELP[18] = {
   "opinion). In this case, non-free-form.",
 };
 
-const char * const GOTOBOOKMARK_HELP[13] = {
-  "Syntax: GotoBookmark [N|-]",
+const char * const GOTOBOOKMARK_HELP[15] = {
+  "Syntax: GotoBookmark [N|+1|-1|-]",
   "Abbreviation: GBM",
   "",
-  "moves the cursor to the Nth or - automatic bookmark, assuming that",
-  "bookmark has been set; see `SetBookmark'. If the optional N",
-  "argument is not specified, it is assumed to be zero. When successful,",
-  "the `-' automatic bookmark is set to the position in the document from",
-  "which the command was issued, so that `GotoBookmark -' returns you to",
-  "the location from which you last issued a `GotoBookmark' command.",
-  "Subsequent repeated `GotoBookmark -' commands toggle you between the",
-  "two locations. Each document's valid bookmark designations are 0 to 9,",
-  "and the `-' automatic bookmark.",
+  "moves the cursor to the designated bookmark if that bookmark is set;",
+  "see `SetBookmark'. Each document has 10 available bookmarks",
+  "designated `0' to `9', plus the automatic bookmark designated by `-'.",
+  "If no option is given, `0' is assumed. The optons `+1' and `-1'",
+  "indicate respectively the next and previous set bookmarks, so that",
+  "repeated `GotoBookmark +1' commands will cycle through all currently",
+  "set bookmarks. When successful, the `-' automatic bookmark is set to",
+  "the position in the document from which the command was issued, so that",
+  "`GotoBookmark -' returns you to the location from which you last issued",
+  "a successful `GotoBookmark' command. Subsequent repeated `GotoBookmark",
+  "-' commands will toggle you between the two locations.",
 };
 
 const char * const GOTOCOLUMN_HELP[8] = {
@@ -550,8 +593,8 @@ const char * const GOTOMARK_HELP[9] = {
   "moves the cursor to the current mark, if it exists. See `Mark'.",
   "",
   "   `GotoMark' is mainly useful if you forgot where you started marking.",
-  "If you want to record a position in a file and jump to it later, you",
-  "may want to use a bookmark. See `SetBookmark'.",
+  "If you want to record positions in a file and jump to them later, you",
+  "may want to use a bookmarks. See `SetBookmark'.",
 };
 
 const char * const HELP_HELP[15] = {
@@ -736,32 +779,41 @@ const char * const MACRO_HELP[32] = {
   "repeatedly.",
 };
 
-const char * const MARK_HELP[14] = {
+const char * const MARK_HELP[12] = {
   "Syntax: Mark [0|1]",
   "Abbreviation: M",
   "",
   "sets the mark at the current position or cancels the previous mark.",
-  "The mark can then be used to perform clip operations. The clip commands",
-  "act on the characters lying between the mark and the cursor.",
+  "The mark and cursor together define the range of text over which clips",
+  "(`Cut', `Copy', `Erase') and left and right shifts operate .",
   "",
-  "   If you invoke `Mark' with no arguments, it will toggle the mark. If",
-  "you specify 0 or 1, the mark will be canceled or set to the current",
+  "   If you invoke `Mark' with no arguments, it will set the mark. If you",
+  "specify 0 or 1, the mark will be canceled or set to the current",
   "position, respectively.  A capital `M' appears on the status bar, if",
   "the mark is active.",
-  "",
-  "   See `Record', for the reason the mark is implemented as a flag.",
 };
 
-const char * const MARKVERT_HELP[10] = {
+const char * const MARKVERT_HELP[21] = {
   "Syntax: MarkVert [0|1]",
   "Abbreviation: MV",
   "",
-  "is the same as `Mark', but the mark is interpreted as vertical by the",
-  "clip handling commands. This means that the region manipulated by the",
-  "cut/paste commands is the rectangle having as vertices the cursor and",
-  "the mark.  Moreover, a capital `V', rather than a capital `M', will",
-  "appear on the status bar. Vertical cut/paste operations are useful for",
-  "handling structured program indentation.",
+  "is the same as `Mark', but the region manipulated by the cut/paste",
+  "commands is the rectangle having as vertices the cursor and the mark.",
+  "If you invoke `MarkVert' with no arguments, it will set the mark. If you",
+  "specify 0 or 1, the mark will be canceled or set to the current",
+  "position, respectively. Moreover, a capital `V', rather than a capital",
+  "`M', will appear on the status bar.",
+  "",
+  "   For example, if you have the following text:",
+  "        aaaBbbccc",
+  "        aaabbbccc",
+  "        aaabbbCcc",
+  "   and you set a vertical mark at `B' then move the cursor to `C', you",
+  "can cut or copy all of the `B's.",
+  "",
+  "   If you have made a vertical cut or copy, it's very likely you will",
+  "want to use `PasteVert' rather than the usual `Paste' to reinsert the",
+  "text in a rectangle. See `PasteVert'.",
 };
 
 const char * const MATCHBRACKET_HELP[8] = {
@@ -771,7 +823,7 @@ const char * const MATCHBRACKET_HELP[8] = {
   "moves the cursor to the bracket associated with the bracket the cursor",
   "is on. If the cursor is not on a bracket, or there is no bracket",
   "associated with the current one, an error message is issued. Recognized",
-  "brackets are `{}', `()', `[]' and `<>'.",
+  "brackets are `{}', `()', `[]' and `<>'.  See `AutoMatchBracket'.",
 };
 
 const char * const MODIFIED_HELP[15] = {
@@ -1093,11 +1145,11 @@ const char * const POPPREFS_HELP[24] = {
   "",
   "PopPrefs restores the following values from the preferences stack:",
   "",
-  "       AutoIndent   DoUndo      PreserveCR   Tabs",
-  "       AutoPrefs    FreeForm    ReadOnly     TabSize",
-  "       Binary       HexCode     RightMargin  UTF8Auto",
-  "       CaseSearch   Insert      SearchBack   VisualBell",
-  "       ClipNumber   NoFileReq   StatusBar    WordWrap",
+  "       AutoIndent   DoUndo      PreserveCR   ShiftTabs   WordWrap",
+  "       AutoPrefs    FreeForm    ReadOnly     Tabs",
+  "       Binary       HexCode     RightMargin  TabSize",
+  "       CaseSearch   Insert      SearchBack   UTF8Auto",
+  "       ClipNumber   NoFileReq   StatusBar    VisualBell",
 };
 
 const char * const PRESERVECR_HELP[15] = {
@@ -1163,11 +1215,11 @@ const char * const PUSHPREFS_HELP[24] = {
   "",
   "`PushPrefs' saves the following values on the preferences stack:",
   "",
-  "       AutoIndent   DoUndo      PreserveCR   Tabs",
-  "       AutoPrefs    FreeForm    ReadOnly     TabSize",
-  "       Binary       HexCode     RightMargin  UTF8Auto",
-  "       CaseSearch   Insert      SearchBack   VisualBell",
-  "       ClipNumber   NoFileReq   StatusBar    WordWrap",
+  "       AutoIndent   DoUndo      PreserveCR   ShiftTabs   WordWrap",
+  "       AutoPrefs    FreeForm    ReadOnly     Tabs",
+  "       Binary       HexCode     RightMargin  TabSize",
+  "       CaseSearch   Insert      SearchBack   UTF8Auto",
+  "       ClipNumber   NoFileReq   StatusBar    VisualBell",
 };
 
 const char * const QUIT_HELP[6] = {
@@ -1195,7 +1247,7 @@ const char * const READONLY_HELP[13] = {
 
 const char * const RECORD_HELP[21] = {
   "Syntax: Record [0|1]",
-  "Abbreviation: Rec",
+  "Abbreviation: REC",
   "",
   "sets the recording state flag. When this flag becomes true, `ne' starts",
   "recording your actions in a new macro. When it becomes false, the macro",
@@ -1502,16 +1554,51 @@ const char * const SELECTDOC_HELP[14] = {
   "should be enough. See `NextDoc', and `PrevDoc'.",
 };
 
-const char * const SETBOOKMARK_HELP[10] = {
-  "Syntax: SetBookmark [N|-]",
+const char * const SETBOOKMARK_HELP[12] = {
+  "Syntax: SetBookmark [N|+1|-1|-]",
   "Abbreviation: SBM",
   "",
-  "sets the Nth bookmark to the current cursor position. If the optional N",
-  "argument is not specified, it is assumed to be zero. You can also unset",
-  "the `-' automatic bookmark, but it will be reset automatically to the",
-  "current position whenever a `GotoBookmark' command is issued. Each",
-  "document's valid bookmark designations are 0 to 9, and the `-' automatic",
-  "bookmark.",
+  "sets a document bookmark to the current cursor position.  Each document",
+  "has 10 available bookmarks designated `0' to `9', plus the automatic",
+  "bookmark designated by `-'. If no option is given, `0' is assumed.",
+  "Values of N from `0' to `9' set the Nth bookmark, while `+1' and `-1'",
+  "indicate respectively the next and previous available unset bookmarks.",
+  "You can also set the `-' automatic bookmark, but it will be reset",
+  "automatically to the current position whenever a `GotoBookmark' command",
+  "is issued.",
+};
+
+const char * const SHIFT_HELP[22] = {
+  "Syntax: Shift [<|>][n][t|s]]",
+  "Abbreviation: SH",
+  "",
+  "shifts the text on lines between the mark and the cursor either right",
+  "(`>', the default) or left (`<') by adding or removing white space on",
+  "each line. The adjustment size, specified as an unsigned integer `n',",
+  "is in units of the current tab size (`t') or spaces (`s'). The default",
+  "is 1. Adjustments start at the left edge of a vertical mark, or column",
+  "1 otherwise. If the mark is not currently set, only the current line is",
+  "affected.",
+  "",
+  "   `Shift' will insert tab characters only if the document's `Tabs'",
+  "flag is set--in which case a `t' appears in the status bar-- and the",
+  "`ShiftTabs' is also set--in which case the `t' in the status bar will",
+  "be upper case. If either of the `ShiftTabs' or `Tabs' flags are off",
+  "(and therefore the `t' in the status bar is lower case or missing,",
+  "respectively) `Shift' will only insert spaces.",
+  "",
+  "   In the case of left shifts, if any indicated line has insufficient",
+  "leading white space for the requested adjustment to be made, then",
+  "`Shift' reports an error and makes no changes.",
+};
+
+const char * const SHIFTTABS_HELP[7] = {
+  "Syntax: ShiftTabs [0|1]",
+  "Abbreviation: SHT",
+  "",
+  "sets the `ShiftTabs' flag. When this flag and the `Tabs' flag are both",
+  "true, left and right `Shift' commands may use tab characters to adjust",
+  "leading white space. Otherwise only spaces are used.  `Shift'.",
 };
 
 const char * const STATUSBAR_HELP[23] = {
@@ -1547,7 +1634,7 @@ const char * const SUSPEND_HELP[6] = {
   "command `fg' is used to resume `ne'.",
 };
 
-const char * const SYNTAX_HELP[42] = {
+const char * const SYNTAX_HELP[43] = {
   "Syntax: Syntax [NAME|*]",
   "Abbreviation: SY",
   "",
@@ -1586,6 +1673,7 @@ const char * const SYNTAX_HELP[42] = {
   "            profile, rc",
   "        skill: il",
   "        tex: latex, dtx, sty",
+  "        texinfo: texi, txi",
   "        troff: 1",
   "        verilog: v, vh, vhd",
   "        xml: xsd",
@@ -1778,16 +1866,17 @@ const char * const UNLOADMACROS_HELP[12] = {
   "version available.",
 };
 
-const char * const UNSETBOOKMARK_HELP[10] = {
-  "Syntax: UnsetBookmark [N|-]",
+const char * const UNSETBOOKMARK_HELP[11] = {
+  "Syntax: UnsetBookmark [N|+1|-1|-|*]",
   "Abbreviation: UBM",
   "",
-  "unsets the Nth bookmark, making it as if it had never been set; see",
-  "`SetBookmark'. If the optional N argument is not specified, it is",
-  "assumed to be zero. You can also unset the `-' automatic bookmark, but",
-  "it will be reset automatically to the current position whenever a",
-  "`GotoBookmark' command is issued. Each document's valid bookmark",
-  "designations are 0 to 9, and the  `-' automatic bookmark.",
+  "unsets either the Nth bookmark, the next (+1) or previous (-1) set",
+  "bookmarks, the automatic (-) bookmark, or all (*) bookmarks, making it",
+  "as if they had never been set; see `SetBookmark'. If no option is",
+  "specified, N is assumed to be zero. While you can unset the automatic",
+  "bookmark `-', it will be reset automatically to the current position",
+  "whenever a `GotoBookmark' command is issued. Each document's valid",
+  "bookmark designations are 0 to 9, and the  `-' automatic bookmark.",
 };
 
 const char * const UTF8_HELP[20] = {
