@@ -1,7 +1,7 @@
 /* Help string externs.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -22,6 +22,7 @@
 extern const char * const ABOUT_HELP[ 7 ];
 extern const char * const ADJUSTVIEW_HELP[ 26 ];
 extern const char * const ALERT_HELP[ 5 ];
+extern const char * const ATOMICUNDO_HELP[ 22 ];
 extern const char * const AUTOCOMPLETE_HELP[ 20 ];
 extern const char * const AUTOINDENT_HELP[ 22 ];
 extern const char * const AUTOMATCHBRACKET_HELP[ 11 ];
@@ -43,7 +44,7 @@ extern const char * const DELETEEOL_HELP[ 10 ];
 extern const char * const DELETELINE_HELP[ 9 ];
 extern const char * const DELETENEXTWORD_HELP[ 6 ];
 extern const char * const DELETEPREVWORD_HELP[ 8 ];
-extern const char * const DOUNDO_HELP[ 20 ];
+extern const char * const DOUNDO_HELP[ 21 ];
 extern const char * const ERASE_HELP[ 6 ];
 extern const char * const ESCAPE_HELP[ 7 ];
 extern const char * const ESCAPETIME_HELP[ 16 ];
@@ -52,7 +53,7 @@ extern const char * const EXIT_HELP[ 6 ];
 extern const char * const FASTGUI_HELP[ 19 ];
 extern const char * const FIND_HELP[ 12 ];
 extern const char * const FINDREGEXP_HELP[ 12 ];
-extern const char * const FLAGS_HELP[ 32 ];
+extern const char * const FLAGS_HELP[ 33 ];
 extern const char * const FLASH_HELP[ 6 ];
 extern const char * const FREEFORM_HELP[ 18 ];
 extern const char * const GOTOBOOKMARK_HELP[ 15 ];
@@ -72,8 +73,8 @@ extern const char * const LINEUP_HELP[ 6 ];
 extern const char * const LOADAUTOPREFS_HELP[ 7 ];
 extern const char * const LOADPREFS_HELP[ 16 ];
 extern const char * const MACRO_HELP[ 32 ];
-extern const char * const MARK_HELP[ 14 ];
-extern const char * const MARKVERT_HELP[ 10 ];
+extern const char * const MARK_HELP[ 12 ];
+extern const char * const MARKVERT_HELP[ 21 ];
 extern const char * const MATCHBRACKET_HELP[ 8 ];
 extern const char * const MODIFIED_HELP[ 15 ];
 extern const char * const MOVEBOS_HELP[ 6 ];
@@ -130,9 +131,11 @@ extern const char * const SAVEPREFS_HELP[ 11 ];
 extern const char * const SEARCHBACK_HELP[ 14 ];
 extern const char * const SELECTDOC_HELP[ 14 ];
 extern const char * const SETBOOKMARK_HELP[ 12 ];
+extern const char * const SHIFT_HELP[ 22 ];
+extern const char * const SHIFTTABS_HELP[ 7 ];
 extern const char * const STATUSBAR_HELP[ 23 ];
 extern const char * const SUSPEND_HELP[ 6 ];
-extern const char * const SYNTAX_HELP[ 42 ];
+extern const char * const SYNTAX_HELP[ 43 ];
 extern const char * const SYSTEM_HELP[ 10 ];
 extern const char * const TABS_HELP[ 24 ];
 extern const char * const TABSIZE_HELP[ 9 ];
