@@ -13,7 +13,7 @@ $| = 1;
 
 my $copyright =
   qq[	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -336,6 +336,10 @@ $copyright  */
 
 #include "ne.h"
 
+/* This file is generated by info2src.pl from data found in ne.texinfo.
+   Changes made directly to this file will we lost when the documentation
+   is updated. */
+
 /* Here we have the name of all commands. They are extern'd in names.h. Note
 that whenever you add or remove a command, you should immediately change also
 the command_names array at the end of this file. */
@@ -378,6 +382,7 @@ sub tail_names_c
 
 const char PLAYONCE_ABBREV[] = "PL 1";
 const char MIDDLEVIEW_ABBREV[] = "AV M";
+const char SHIFTLEFT_ABBREV[] = "SH <";
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
@@ -423,6 +428,7 @@ sub tail_names_h
 /* These are extras that are very useful in the default menus and key bindings. */
 extern const char PLAYONCE_ABBREV[];
 extern const char MIDDLEVIEW_ABBREV[];
+extern const char SHIFTLEFT_ABBREV[];
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
