@@ -1,7 +1,7 @@
 /* Input class, key bindings and upper casing vector definitions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -497,3 +497,290 @@ const char meta_prefixed[128][3] = {
 	{ '\x1b', 126 },
 	{ '\x1b', 127 }
 };
+
+const char *key_stroke[NUM_KEYS] = {
+
+	/* Control-letter bindings (Ctrl-X)*  0..31  0..1f */
+
+	"^@",
+	"^A",
+	"^B",
+	"^C",
+	"^D",      
+	"^E",      
+	"^F",      
+	"^G",      
+
+	"^H",      
+	"^I",      
+	"^J",      
+	"^K",      
+	"^L",      
+	"^M",      
+	"^N", 
+	"^O",      
+
+	"^P", 
+	"^Q", 
+	"^R", 
+	"^S", 
+	"^T", 
+	"^U", 
+	"^V", 
+	"^W", 
+
+	"^X", 
+	"^Y", 
+	"^Z", 
+	"^[", 
+	"^\\", 
+	"^]", 
+	"^^", 
+	"^_", 
+       
+   /* These keys map to strings, not commands. 31..126  20..7e */
+	" ", "!", "\"","#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", "/",   /* space ! " # $ % & ' ( ) * + , - . /  */
+	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?",   /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
+   
+	"@", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O",   /* @ A B C D E F G H I J K L M N O  */ 
+	"P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\","]", "^", "_",   /* P Q R S T U V W X Y Z [ \ ] ^ _  */
+	"`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o",   /* ` a b c d e f g h i j k l m n o  */
+	"p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~",        /* p q r s t u v w x y z { | } ~    */
+   "Del",                         /*  127  7f */
+
+	/* Control-meta-letter bindings (Ctrl-Alt-X)  128..159 80..9f */
+
+	"^[@",
+	"^[A",
+	"^[B",
+	"^[C",
+	"^[D",
+	"^[E",
+	"^[F",
+	"^[G ",
+	"^[H",
+	"^[I ",
+	"^[J",
+	"^[K",
+	"^[L",
+	"^[M",
+	"^[N",
+	"^[O",
+
+	"^[P",
+	"^[Q",
+	"^[R",
+	"^[S",
+	"^[T",
+	"^[U",
+	"^[V",
+	"^[W",
+	"^[X",
+	"^[Y",
+	"^[Z",
+	"^[[",
+	"^[\\",
+	"^[]",
+	"^[^",
+	"^[_",
+
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 160..175  A0..AF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 176..191  B0..BF */
+
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 192..207  C0..CF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 207..223  D0..DF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 224..239  E0..EF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 240..255  F0..FF */
+
+
+	/* The following bindings are for the terminfo extended codes (see keycodes.h). */
+
+	/* Cursor movement keys 256..271  100..10F */
+	"", 
+	"Up",
+	"Down",
+	"Left",
+	"Right",
+	"IncUp",
+	"IncDn",
+	"PgDn",
+	"PgUp",
+	"LnDn",
+	"LnUp",
+	"", 
+	"", 
+	"", 
+	"", 
+	"",
+
+	/* Editing keys   272..287   110..11F */
+	"Del",
+	"", 
+	"BackSp",
+	"DelLn",
+	"UndelLn",
+	"DelCh",
+	"Ins",
+	"",
+	"Clear",
+	"", 
+	"", 
+	"", 
+	"", 
+	"", 
+	"", 
+	"",
+
+	/* Keypad keys and fake (simulated) menu key  288..303  120..12F */
+	"KPSoF",
+	"KPPrPg",
+	"S/EoL",
+	"EOF",
+	"NxtPg",
+	"Exec",
+	"", 
+	"",
+	"", 
+	"", 
+	"", 
+	"", 
+	"", 
+	"", 
+	"", 
+	"",
+
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 304..319  130..13F */
+
+	/* Function keys   320..335  140..14F  */
+	"F0",
+	"F1",
+	"F2",
+	"F3",
+	"F4",
+	"F5",
+	"F6",
+	"F7",
+	"F8",
+	"F9",
+	"F10",
+	"F11",
+	"F12",
+	"F13",
+	"F14",
+	"F15",
+
+	"F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23", "F24", "F25", "F26", "F27", "F28", "F29", "F30", "F31",  /* 336..351  150..15F */
+	"F32", "F33", "F34", "F35", "F36", "F37", "F38", "F39", "F40", "F41", "F42", "F43", "F44", "F45", "F46", "F47",  /* 352..367  160..16F */
+	"F48", "F49", "F50", "F51", "F52", "F53", "F54", "F55", "F56", "F57", "F58", "F59", "F60", "F61", "F62", "F63",  /* 368..383  170..17F */
+
+	/* Prefix-simulated META (ESC Ctrl-a)  384..416  180..1A0 */
+
+	"ESC-^@",
+	"ESC-^A",
+	"ESC-^B",
+	"ESC-^C",
+	"ESC-^D",
+	"ESC-^E",
+	"ESC-^F",
+	"ESC-^G",
+	"ESC-^H",
+	"ESC-^I",
+	"ESC-^J",
+	"ESC-^K",
+	"ESC-^L",
+	"ESC-^M",
+	"ESC-^N",
+	"ESC-^O",
+	
+	"ESC-^P",
+	"ESC-^Q",
+	"ESC-^R",
+	"ESC-^S",
+	"ESC-^T",
+	"ESC-^U",
+	"ESC-^V",
+	"ESC-^W",
+	"ESC-^X",
+	"ESC-^Y",
+	"ESC-^Z",
+	"ESC-^[",
+	"ESC-^\\",
+	"ESC-^]",
+	"ESC-^^",
+	"ESC-^_",
+
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 416..431  1A0..1AF */
+	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 432..447  1B0..1BF */
+
+   /* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
+	"ESC-@",
+	"ESC-A",
+	"ESC-B",
+	"ESC-C",
+	"ESC-D",
+	"ESC-E",
+	"ESC-F",
+	"ESC-G",
+	"ESC-H",
+	"ESC-I",
+	"ESC-J",
+	"ESC-K",
+	"ESC-L",
+	"ESC-M",
+	"ESC-N",
+	"ESC-O",
+
+	"ESC-P",
+	"ESC-Q",
+	"ESC-R",
+	"ESC-S",
+	"ESC-T",
+	"ESC-U",
+	"ESC-V",
+	"ESC-W",
+	"ESC-X",
+	"ESC-Y",
+	"ESC-Z",
+	"ESC-[",
+	"ESC-\\",
+	"ESC-]",
+	"ESC-^",
+	"ESC-_",
+
+
+   /* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
+	"ESC-@",
+	"ESC-a",
+	"ESC-b",
+	"ESC-c",
+	"ESC-d",
+	"ESC-e",
+	"ESC-f",
+	"ESC-g",
+	"ESC-h",
+	"ESC-i",
+	"ESC-j",
+	"ESC-k",
+	"ESC-l",
+	"ESC-m",
+	"ESC-n",
+	"ESC-o",
+
+	"ESC-p",
+	"ESC-q",
+	"ESC-r",
+	"ESC-s",
+	"ESC-t",
+	"ESC-u",
+	"ESC-v",
+	"ESC-w",
+	"ESC-x",
+	"ESC-y",
+	"ESC-z",
+	"ESC-[",
+	"ESC-\\",
+	"ESC-]",
+	"ESC-^",
+	"ESC-_",
+
+};
