@@ -1,7 +1,7 @@
 /* Menu handling function. Includes also key and menu configuration parsing.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -169,6 +169,8 @@ static const menu_item extras_item[] =
 		{ "Undo      f5/[U", UNDO_ABBREV },
 		{ "Redo      f6/[R", REDO_ABBREV },
 		{ "Center         ", CENTER_ABBREV },
+		{ "Shift Right    ", SHIFT_ABBREV },
+		{ "Shift Left     ", SHIFTLEFT_ABBREV },
 		{ "Paragraph    [P", PARAGRAPH_ABBREV },
 		{ "Adjust View  ^^", ADJUSTVIEW_ABBREV },
 		{ "Middle View  [C", MIDDLEVIEW_ABBREV },
@@ -577,9 +579,9 @@ char *gen_flag_string(const buffer * const b) {
 	string[i++] = b->opt.free_form      ? 'f' : '-';
 	string[i++] = b->opt.auto_prefs     ? 'p' : '-';
 	string[i++] = verbose_macros        ? 'v' : '-';
-	string[i++] = b->opt.do_undo        ? 'u' : '-';
+	string[i++] = b->opt.do_undo        ? (b->atomic_undo ? 'U' : 'u') : '-';
 	string[i++] = b->opt.read_only      ? 'r' : '-';
-	string[i++] = b->opt.tabs           ? 't' : '-';
+	string[i++] = b->opt.tabs           ? (b->opt.shift_tabs ? 'T' : 't' ) : '-';
 	string[i++] = b->opt.binary         ? 'B' : '-';
 	string[i++] = b->marking            ? (b->mark_is_vertical ? 'V' :'M') : '-';
 	string[i++] = b->recording          ? 'R' : '-';
