@@ -1,24 +1,22 @@
 /* Navigation functions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+	This library is free software; you can redistribute it and/or modify it
+	under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 3 of the License, or (at your
+	option) any later version.
+
+	This library is distributed in the hope that it will be useful, but
+	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -39,7 +37,7 @@ in an obvious way. (Yeah, right.) */
 
 /*********************************************************************************
 You stand absolutely no chance of understanding this code if you aren't intimately
-familiar with this diagram.      
+familiar with this diagram.
 
           |<- cur_pos (in bytes)-->|
           |<- cur_char (in chars)->|
@@ -56,7 +54,7 @@ familiar with this diagram.
     cur_y |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
-     ---  |         |              @ <-Cursor |          | ---        |      ne_lines   
+     ---  |         |              @ <-Cursor |          | ---        |      ne_lines
           |         |                         |          |          num_lines |
           |         |                         |          |            |       |
           |         |                         |          |            |       |
@@ -68,10 +66,10 @@ familiar with this diagram.
           +----------------------------------------------+          -----
 
                     |<---- ne_columns ------->|
-         
+
 ************************************************************************************/
 
-          
+
 /* "Resyncs" cur_pos (the current character the cursor is on) with cur_x and
 	win_x. It has to take into account the TAB expansion, and can cause
 	left/right movement in order to properly land on a real character.  x is the
@@ -225,21 +223,27 @@ int line_down(buffer * const b) {
  * because the screen may have been resized since the last time we
  * were here.
  */
- 
+
 void keep_cursor_on_screen(buffer * const b) {
+   int shift_right;
+	b->opt.tab_size = min(b->opt.tab_size, max(ne_columns / 2 - 1,1));
+	if (shift_right = b->win_x % b->opt.tab_size) {
+		b->win_x -= shift_right;
+		b->cur_x += shift_right;
+	}
 	if (b->cur_y > ne_lines - 2) {
 		while(b->cur_y > ne_lines - 2) {
 			b->cur_y--;
 			b->win_y++;
 			b->attr_len = -1;
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
-		}  
+		}
 		assert(b->win_y = b->cur_line - b->cur_y);
 		b->y_wanted = FALSE;
 	}
 
 	while(b->cur_x >= ne_columns) {
-		b->win_x += b->opt.tab_size; 
+		b->win_x += b->opt.tab_size;
 		b->cur_x -= b->opt.tab_size;
 	}
 }
@@ -469,7 +473,7 @@ int page_up(buffer * const b) {
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 			b->cur_line--;
 		}
-		   
+		
 	   /* Should we shift the view up? */
 		if (b->win_y > 0 /* We aren't already at the top */
 				&& b->win_y + b->wanted_cur_y > b->wanted_y) { /* Gap between virtual cursor and TOS is to small */
@@ -505,7 +509,7 @@ int page_down(buffer * const b) {
    }
 
    shift_view = (b->win_y + disp < b->num_lines); /* can't already see the last line */
-   
+
 	for (i = 0; i < disp; i++) {
 		b->wanted_y++; /* We want to move down */
 			
@@ -515,7 +519,7 @@ int page_down(buffer * const b) {
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 			b->cur_line++;
 		}
-		   
+		
 	   /* Should we shift the view down? */
 		if (shift_view /* already decided we should */
 				&& b->wanted_y - b->wanted_cur_y >  b->win_y) { /* Gap between virtual cursor and TOS is to big */
@@ -587,7 +591,7 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 	b->y_wanted = 0;
 
 	if (!p) p = "t";
-	  
+	
 	while(*p) {
 		disp = 0;
 		mag = max(0,strtol(p+1, &q, 0));
@@ -642,7 +646,7 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 			case 'R' :
 				/* Shift the view as far right as possible, or mag columns. */
 				if (mag == 0) mag = b->win_x;
-				while (b->cur_x < ne_columns - ne_columns % b->opt.tab_size && b->win_x >= b->opt.tab_size && mag > 0) {
+				while (b->cur_x < ne_columns - b->opt.tab_size && b->win_x >= b->opt.tab_size && mag > 0) {
 					mag      -= b->opt.tab_size;
 					b->win_x -= b->opt.tab_size;
 					b->cur_x += b->opt.tab_size;
@@ -771,7 +775,7 @@ void move_to_sol(buffer * const b) {
 	t = b->win_x;
 	b->win_x =
 	b->cur_x =
-	b->cur_pos = 
+	b->cur_pos =
 	b->cur_char = 0;
 
 	if (t &&  b == cur_buffer) update_window(b);
@@ -898,7 +902,6 @@ void move_to_bof(buffer * const b) {
 
 
 void toggle_sof_eof(buffer * const b) {
-
 	if (b->cur_line == 0 && b->cur_pos == 0) {
 		delay_update();
 		move_to_bof(b);
