@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -36,7 +36,7 @@ message disappears as soon as any key is typed. */
 
 char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify",
 										"it under the terms of the GNU General Public License as published by",
@@ -172,9 +172,9 @@ void about(int show) {
 		print_message(ABOUT_MSG);
 	}
 	else {
-      ttysize();
-      keep_cursor_on_screen(cur_buffer);
-      reset_window();
+		ttysize();
+		keep_cursor_on_screen(cur_buffer);
+		reset_window();
 	}
 }
 
@@ -439,7 +439,7 @@ int main(int argc, char **argv) {
 			cur_buffer->automatch.shown = 0;
 		}
 
-		if ( c == INVALID_CHAR ) continue; // Window resizing.
+		if ( c == INVALID_CHAR ) continue; /* Window resizing. */
 		ic = CHAR_CLASS(c);
 
 		if (displaying_info) {
