@@ -1,7 +1,7 @@
 /* Function prototypes
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -125,6 +125,7 @@ int paragraph(buffer *b);
 int center(buffer *b);
 int auto_indent_line(buffer * const b, const int line, line_desc * const ld, const int up_to_col);
 int backtab(buffer *b);
+int shift(buffer *b, char *p, char *msg, int msg_size);
 
 /* errors.c */
 
@@ -294,6 +295,7 @@ int get_char_width(const unsigned char * const s, const encoding_type encoding);
 int ne_ispunct(const int c, const int encoding);
 int ne_isspace(const int c, const int encoding);
 int ne_isword(const int c, const int encoding);
+int context_prefix(const buffer *b, unsigned char **p, int *prefix_pos, encoding_type encoding);
 
 /* term.c */
 int output_width(int c);
@@ -327,6 +329,7 @@ void term_init(void);
 void start_undo_chain(buffer *b);
 void end_undo_chain(buffer *b);
 int add_undo_step(buffer *b, int line, int pos, int len);
+void fix_last_undo_step(buffer *b, int delta);
 int add_to_undo_stream(undo_buffer *ub, const char *p, int len);
 void reset_undo_buffer(undo_buffer *ub);
 int undo(buffer *b);
