@@ -1,7 +1,7 @@
 /* Requester handling.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2011 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -375,6 +375,8 @@ int request_strings(const char * const * const _entries, const int _num_entries,
 						request_toggle_seof();
 						break;
 
+					case CLOSEDOC_A: 
+					case QUIT_A:
 					case ESCAPE_A:
 						return -1;
 					}
