@@ -1,24 +1,22 @@
 /* Miscellaneous support functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2010 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+	This library is free software; you can redistribute it and/or modify it
+	under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 3 of the License, or (at your
+	option) any later version.
+
+	This library is distributed in the hope that it will be useful, but
+	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+	for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -558,3 +556,25 @@ int ne_isspace(const int c, const int encoding) {
 int ne_isword(const int c, const int encoding) {
 	return c == '_' || !(ne_isspace(c, encoding) || ne_ispunct(c, encoding));
 }
+
+/* Returns a copy of the ne_isword() string to the left of the cursor, or an empty
+	string if none is found. Consumer is responsible for seeing that the string is freed.
+	*prefix_pos is the offset from the beginning of the current line where the prefix starts. */
+int context_prefix(const buffer *b, unsigned char **p, int *prefix_pos, encoding_type encoding) {
+
+	*prefix_pos = b->cur_pos;
+	if (*prefix_pos && *prefix_pos <= b->cur_line_desc->line_len) {
+		*prefix_pos = prev_pos(b->cur_line_desc->line, *prefix_pos, b->encoding);
+		while (*prefix_pos && ne_isword(get_char(&b->cur_line_desc->line[*prefix_pos], b->encoding), b->encoding))
+			*prefix_pos = prev_pos(b->cur_line_desc->line, *prefix_pos, b->encoding);
+		if (! ne_isword(get_char(&b->cur_line_desc->line[*prefix_pos], b->encoding), b->encoding))
+			*prefix_pos = next_pos(b->cur_line_desc->line, *prefix_pos, b->encoding);
+		*p = malloc(b->cur_pos + 1 - *prefix_pos);
+		if (!*p) return OUT_OF_MEMORY;
+		strncpy(*p, &b->cur_line_desc->line[*prefix_pos], b->cur_pos - *prefix_pos);
+	} 
+	else *p = malloc(1); /* no prefix left of the cursor; we'll give an empty one. */
+	
+	(*p)[b->cur_pos - *prefix_pos] = 0;
+	return OK;
+}
